function score = evaluate_with_test( obj, test, groundtruth, codebook )
% Evaluation of learned model using test data
% Input
%   test            dim * test_datanum test data matrix
%   groundtruth     test_datanum vector of ground truth label
%   codebook        dim * k matrix of PQ codebooks
%                   if empty, samples are regarded as linear feature vector
% Output
%   score           classification accuracy [0,1]
%                   score.avg is computed by averaged classifier
%                   score.nonavg is computed by not averaged classifier
%
% Copyright (c) 2014, Yoshitaka Ushiku
% This software is released under the MIT License.
% http://opensource.org/licenses/mit-license.php

if nargin < 4 || isempty(codebook)

    % predicting labels (no PQ)
    try
        % try to predict labels by multiplying matrices
        [~,pred_class_avg] = max([test; ones(1,length(groundtruth))]'*obj.avgmu,[],2);
        [~,pred_class_nonavg] = max([test; ones(1,length(groundtruth))]'*obj.mu,[],2);
    catch me
        % if matrix multiplication fails (mainly due to memory shortage)
        disp(me)
        pred_class_avg = zeros(length(groundtruth),1);
        pred_class_nonavg = zeros(length(groundtruth),1);
        for tstind = 1:length(groundtruth)
            fprintf('test %d/%d \033[K\r',tstind,length(groundtruth));
            F = [test(:,tstind);1];
            [~,pred_class_avg(tstind)] = max(F'*obj.avgmu);
            [~,pred_class_nonavg(tstind)] = max(F'*obj.mu);
        end
    end

else

    % predicting labels (PQ)
    d = size(codebook,1);
    G = d / size(test,1);
    pred_class_avg = zeros(length(groundtruth),1);
    pred_class_nonavg = zeros(length(groundtruth),1);
    for tstind = 1:length(groundtruth)
        fprintf('test %d/%d \033[K\r',tstind,length(groundtruth));        
        % decoding PQ
        cb_ind = double(reshape(test(:,tstind*ones(G,1))',d,1))*d + (1:d)';
        F = [codebook(cb_ind); 1];
        % predicting
        [~,pred_class_avg(tstind)] = max(F'*obj.avgmu);
        [~,pred_class_nonavg(tstind)] = max(F'*obj.mu);
    end
    
end

% evaluating predicted labels
score.avg = sum(groundtruth(:)==pred_class_avg(:))/length(groundtruth);
score.nonavg = sum(groundtruth(:)==pred_class_nonavg(:))/length(groundtruth);

end
