function score = fix_evaluation( obj, iternum, count, code_train, code_test, labels_train, labels_test, codebook, cache_path )
% Do training multiple iteration and multiple independent times and computes and displays classification accuracy. Accuracy is computed for each final iteration and then mean and standard deviation are computed through all independent learning
% Input
%   iternum         the number of iteration per one sequential learning
%   count           the number of independent learning (total iternum * count iterations are performed)
%   code_train      dim * datanum training matrix
%   code_test       dim * test_datanum test data matrix
%   labels_train    datanum vector or classnum * datanum logical matrix
%   labels_test     test_datanum vector of ground truth label
%   codebook        dim * k matrix of PQ codebooks
%                   if empty, samples are regarded as linear feature vector
%   cache_path      (optional) prefix of model cache file name
% Output
%   score           statistics of classification accuracy through learning of count times
%                   score.avg[mean,std] is accuracy mean and standard deviation of averaged classifier
%                   score.nonavg[mean,std] is accuracy mean and standard deviation of not averaged classifier
%
% Copyright (c) 2014, Yoshitaka Ushiku
% This software is released under the MIT License.
% http://opensource.org/licenses/mit-license.php

if nargin < 8 || isempty(codebook)
    codebook = [];
end
if nargin < 9
    iscache = false;
else
    iscache = true;
end

scores.avg = zeros(1,count);
scores.nonavg = zeros(1,count);
for ind = 1:count
    if iscache
        % try to load the latest classifiers
        cur_cache_path = [cache_path '_' num2str(ind)];
        if exist([cur_cache_path '_iter' num2str(iternum) '.mat'],'file')
            load([cur_cache_path '_iter' num2str(iternum) '.mat']);
        else
            % learn now
            obj = obj.learn(iternum,code_train,labels_train,codebook,[],[],cur_cache_path);
        end
    else
        obj = obj.learn(iternum,code_train,labels_train,codebook);
    end
    % evaluating
    score = obj.evaluate_with_test(code_test,labels_test);
    scores.avg(ind) = score.avg;
    scores.nonavg(ind) = score.nonavg;
end

% evaluating respectively classifiers w/ averaging and classifiers w/o averaging
score.avg = [mean(100*scores.avg) std(100*scores.avg,1)];
score.nonavg = [mean(100*scores.nonavg) std(100*scores.nonavg,1)];
disp(['Accuracy of ' obj.type ' (Averaged) is ' sprintf('%2.1f',score.avg(1)) ' +/- ' sprintf('%2.1f %%',score.avg(2))])
disp(['Accuracy of ' obj.type ' (non-Averaged) is ' sprintf('%2.1f',score.nonavg(1)) ' +/- ' sprintf('%2.1f %%',score.nonavg(2))])

end
