classdef online_learning<handle
% ONLINE_LEARNING Online learning class for many types of learning methods.
%   obj = online_learning(type,ol_param) prepares for online learning.
%   type is the learning method's name. Currently, following methods can be
%   performed. Obtained weight vectors can be averaged in all methods.
%       Perceptron (MUL/uOVR/wOVR)
%       PA         (MUL/uOVR/wOVR)
%       CW         (MUL/uOVR/wOVR)
%       AROW       (MUL/uOVR/wOVR)
%       NHERD      (MUL/uOVR/wOVR)
%       SCW        (MUL/uOVR/wOVR)
%       SGD+SVM    (MUL/uOVR/wOVR)
%   MUL, uOVR, and wOVR mean multiclass learning, uniform one-vs-the-rest
%   learning, and weighted one-vs-the-rest learning, respectively.
%   To see the list of implemented methods, use
%   ONLINE_LEARNING.selectmethod()
%
%   If ol_param is not available, following parameters are employed.
%       Perceptron (u-OVR)
%           lambda = 0
%           t_zero = 1
%       Perceptron (w-OVR)
%           lambda = 0
%           beta = 32
%           t_zero = 1
%       Perceptron (Multiclass)
%           lambda = 0
%           t_zero = 1
%       PA (u-OVR)
%           C = 0.1
%       PA (w-OVR)
%           C = 0.1
%           beta = 32
%       PA (Multiclass)
%           C = 0.1
%       CW (u-OVR)
%           eta = 0.5 (usually 1 >= eta >= 0.5)
%       CW (w-OVR)
%           eta = 0.5 (usually 1 >= eta >= 0.5)
%           beta = 32
%       CW (Multiclass)
%           eta = 0.5 (usually 1 >= eta >= 0.5)
%       AROW (u-OVR)
%           C = 1
%       AROW (w-OVR)
%           C = 1
%           beta = 32
%       AROW (Multiclass)
%           C = 1
%       NHERD (u-OVR)
%           C = 1
%       NHERD (w-OVR)
%           C = 1
%           beta = 32
%       NHERD (Multiclass)
%           C = 1
%       SCW (u-OVR)
%           C = 1
%           eta = 0.5 (usually 1 >= eta >= 0.5)
%       SCW (w-OVR)
%           C = 1
%           eta = 0.5 (usually 1 >= eta >= 0.5)
%           beta = 32
%       SCW (Multiclass)
%           C = 1
%           eta = 0.5 (usually 1 >= eta >= 0.5)
%       SGD+SVM (u-OVR)
%           lambda = -1 (means 1 / # of training samples)
%           t_zero = 1
%       SGD+SVM (w-OVR)
%           lambda = -1 (means 1 / # of training samples)
%           beta = 32
%           t_zero = 1
%       SGD+SVM (Multiclass)
%           lambda = -1 (means 1 / # of training samples)
%           t_zero = 1
%
%   To perform learning, use ONLINE_LEARNING.learn().
%
%   See also ONLINE_LEARNING/LEARN. 

%   Written by Y. Ushiku
%   Oct. 8, 2012, ISI, UT
%
% Copyright (c) 2014, Yoshitaka Ushiku
% This software is released under the MIT License.
% http://opensource.org/licenses/mit-license.php

    properties(Access=private)
        avgmu
        mu
        sigma
        type
        ol_param
    end

    methods
        function obj = online_learning( type, ol_param )
            if nargin < 2
                ol_param = struct;
            end
            obj.type = type;
            obj.avgmu = [];
            obj.mu = [];
            obj.sigma = [];
            %% Perceptron
            if strcmpi(type,'PerceptronuOVR')
                if ~isfield(ol_param,'lambda')
                    ol_param.lambda = 0;
                end
                if ~isfield(ol_param,'t_zero')
                    ol_param.t_zero = 1;
                end
            end
            if strcmpi(type,'PerceptronwOVR')
                if ~isfield(ol_param,'lambda')
                    ol_param.lambda = 0;
                end
                if ~isfield(ol_param,'t_zero')
                    ol_param.t_zero = 1;
                end
                if ~isfield(ol_param,'beta')
                    ol_param.beta = 32;
                end
            end
            if strcmpi(type,'PerceptronMUL')
                if ~isfield(ol_param,'lambda')
                    ol_param.lambda = 0;
                end
                if ~isfield(ol_param,'t_zero')
                    ol_param.t_zero = 1;
                end
            end

            %% PA
            if strcmpi(type,'PAuOVR')
                if ~isfield(ol_param,'C')
                    ol_param.C = 1;
                end
            end
            if strcmpi(type,'PAwOVR')
                if ~isfield(ol_param,'C')
                    ol_param.C = 1;
                end
                if ~isfield(ol_param,'beta')
                    ol_param.beta = 32;
                end
            end
            if strcmpi(type,'PA')
                if ~isfield(ol_param,'C')
                    ol_param.C = 1;
                end
            end

            %% CW
            if strcmpi(type,'CWuOVR')
                if ~isfield(ol_param,'eta')
                    ol_param.eta = 0.5;
                end
            end
            if strcmpi(type,'CWwOVR')
                if ~isfield(ol_param,'eta')
                    ol_param.eta = 0.5;
                end
                if ~isfield(ol_param,'beta')
                    ol_param.beta = 32;
                end
            end
            if strcmpi(type,'CW')
                if ~isfield(ol_param,'eta')
                    ol_param.eta = 0.5;
                end
            end

            %% AROW
            if strcmpi(type,'AROWuOVR')
                if ~isfield(ol_param,'C')
                    ol_param.C = 1;
                end
            end
            if strcmpi(type,'AROWwOVR')
                if ~isfield(ol_param,'C')
                    ol_param.C = 1;
                end
                if ~isfield(ol_param,'beta')
                    ol_param.beta = 32;
                end
            end
            if strcmpi(type,'AROW')
                if ~isfield(ol_param,'C')
                    ol_param.C = 1;
                end
            end

            %% NHERD
            if strcmpi(type,'NHERDuOVR')
                if ~isfield(ol_param,'C')
                    ol_param.C = 1;
                end
            end
            if strcmpi(type,'NHERDwOVR')
                if ~isfield(ol_param,'C')
                    ol_param.C = 1;
                end
                if ~isfield(ol_param,'beta')
                    ol_param.beta = 32;
                end
            end
            if strcmpi(type,'NHERD')
                if ~isfield(ol_param,'C')
                    ol_param.C = 1;
                end
            end
            
            %% SCW
            if strcmpi(type,'SCWuOVR')
                if ~isfield(ol_param,'C')
                    ol_param.C = 1;
                end
                if ~isfield(ol_param,'eta')
                    ol_param.eta = 0.5;
                end
            end
            if strcmpi(type,'SCWwOVR')
                if ~isfield(ol_param,'C')
                    ol_param.C = 1;
                end
                if ~isfield(ol_param,'eta')
                    ol_param.eta = 0.5;
                end
                if ~isfield(ol_param,'beta')
                    ol_param.beta = 32;
                end
            end
            if strcmpi(type,'SCW')
                if ~isfield(ol_param,'C')
                    ol_param.C = 1;
                end
                if ~isfield(ol_param,'eta')
                    ol_param.eta = 0.5;
                end
            end

            %% SGDSVM
            if strcmpi(type,'SGDSVMuOVR')
                if ~isfield(ol_param,'lambda')
                    ol_param.lambda = -1;
                end
                if ~isfield(ol_param,'t_zero')
                    ol_param.t_zero = 1;
                end
            end
            if strcmpi(type,'SGDSVMwOVR')
                if ~isfield(ol_param,'lambda')
                    ol_param.lambda = -1;
                end
                if ~isfield(ol_param,'t_zero')
                    ol_param.t_zero = 1;
                end
                if ~isfield(ol_param,'beta')
                    ol_param.beta = 32;
                end
            end
            if strcmpi(type,'SGDSVMMUL')
                if ~isfield(ol_param,'lambda')
                    ol_param.lambda = -1;
                end
                if ~isfield(ol_param,'t_zero')
                    ol_param.t_zero = 1;
                end
            end
            
            
            obj.ol_param = ol_param;
        end
        
        function ref_param = getparam(obj)
            ref_param = obj.ol_param;
        end
        
        function ref_classifier = getclassifier(obj)
            ref_classifier.mu       = obj.mu;
            ref_classifier.avgmu    = obj.avgmu;
            ref_classifier.sigma    = obj.sigma;
        end
    end
    
    methods(Static)
        function labels = labvec2labmat(labels)
        % M = online_leanrning.labvec2labmat(V) converts a label vector V 
        % to a c-by-n label matrix M, where c is # of classes and n is the 
        % data amount.
            if length(labels) == numel(labels)
                labels = logical(sparse(labels,1:length(labels),1,max(labels),length(labels)));
            end
        end
        
        function [ trainInd, testInd ] = split_data( trainingsample, labels )
        %SPLIT_DATA Data spliting for categorization
        % trainingsample per each label are selected as trainInd, and the remaining are selected as testInd
        %   [ trainInd, testInd ] = split_data( trainingsample, labels )

            % preprocess
            labels = labels(:);

            % generate indices for each class
            classInd = cell(1,max(labels));
            for cind = 1:length(classInd)
                classInd{cind} = find(labels==cind);
            end

            % divide the whole data into training data and test data
            trainInd = [];
            testInd = [];
            for cind = 1:length(classInd)
                cur_seed = online_learning.randpermrs(length(classInd{cind}));
                trainInd = cat(1,trainInd,classInd{cind}(cur_seed(1:trainingsample)));
                testInd  = cat(1,testInd ,classInd{cind}(cur_seed(trainingsample+1:length(classInd{cind}))));
            end

            % kind preprocess (random permutation of trainInd)
            rp = online_learning.randpermrs(length(trainInd));
            trainInd = trainInd(rp);

        end

        function type = selectmethod(typenum)
        % get learning method type string from the integer id typenum
        % if no arguments are given, the user is requested to select method interactively
            typenames = {'Perceptron (OVR with all samples)',...
                         'Perceptron (Sample-num.-weighted OVR)',...
                         'Perceptron (Multiclass)',...
                         'PA (OVR with all samples)',...
                         'PA (Sample-num.-weighted OVR)',...
                         'PA (Multiclass)',...
                         'CW (OVR with all samples)',...
                         'CW (Sample-num.-weighted OVR)',...
                         'CW (Multiclass)',...
                         'AROW (OVR with all samples)',...
                         'AROW (Sample-num.-weighted OVR)',...
                         'AROW (Multiclass)',...
                         'NHERD (OVR with all samples)',...
                         'NHERD (Sample-num.-weighted OVR)',...
                         'NHERD (Multiclass)',...
                         'SCW (OVR with all samples)',...
                         'SCW (Sample-num.-weighted OVR)',...
                         'SCW (Multiclass)',...
                         'SGD+SVM (OVR with all samples)',...
                         'SGD+SVM (Sample-num.-weighted OVR)',...
                         'SGD+SVM (Multiclass)'};
            methodnames = {'PerceptronuOVR',...
                           'PerceptronwOVR',...
                           'PerceptronMUL',...
                           'PAuOVR',...
                           'PAwOVR',...
                           'PA',...
                           'CWuOVR',...
                           'CWwOVR',...
                           'CW',...
                           'AROWuOVR',...
                           'AROWwOVR',...
                           'AROW',...
                           'NHERDuOVR',...
                           'NHERDwOVR',...
                           'NHERD',...
                           'SCWuOVR',...
                           'SCWwOVR',...
                           'SCW',...
                           'SGDSVMuOVR',...
                           'SGDSVMwOVR',...
                           'SGDSVMMUL'};
            if nargin<1
                disp('Select the type of descriptor you need.')
                for ind = 1:length(typenames)
                    disp(['    ' num2str(ind) ':' typenames{ind}]);
                end
                while true
                    typenum = input('Input the number:');
                    fprintf('\n');
                    disp(['Your selection is ' methodnames{typenum} '.']);
                    isok = input('Input just ENTER if it''s OK:');
                    if isempty(isok)
                        type = methodnames{typenum};
                        break;
                    end
                end
            else
                type = methodnames{typenum};
            end
        end

        function p = randpermrs(n)
            % Surrogate function for randperm.
            % Basically, native randperm() permutates an index in the same order 
            % when this function is called within multiple MATLAB processes.

            rs = clock;
            try
                p = randperm(RandStream.create('mrg32k3a','Seed',rem(rs(6),1)*10000),n);
            catch me
                disp(me)
                rand('state',ceil(rs(6)*10000)-rs(6)*10000);
                [Y,p] = sort(rand(1,n));
            end
            
        end

        % Product Quantization tools
        C = pq_kmeans(X, k, G, iter_lim)
        codebook = pq_learn_codebook(train_samples, pq_k, pq_G, pq_block_size)
        code = pq_compress(sample, codebook, pq_k, pq_G)
    end
    
end

