function scores = split_convergecheck( obj, iternum, count, code, labels, trainingsample, codebook, cache_path, scores )
% Split data randomly into training samples and test samples and evaluate 'count' times
% Data split is changed in each evaluation
% Input
%   iternum         the number of iteration per one sequential learning
%   count           the number of independent learning (total iternum * count iterations are performed)
%   code            dim * datanum data matrix (split into training and test in this function)
%   labels          datanum vector or classnum * datanum logical matrix
%   trainingsample  the number of samples to be used as training samples for each label; remaining samples are used as test
%   codebook        dim * k matrix of PQ codebooks
%                   if empty, samples are regarded as linear feature vector
%   cache_path      (optional) prefix of model cache file name
%   scores          pre-computed classification accuracy matrix
%                   if specified, the iteration which is already exist in scores is skipped
% Output
%   scores          classification accuracy [0,1] for each iterations
%                   scores.avg(iternum_index,count_index) is computed by averaged classifier
%                   scores.nonavg(iternum_index,count_index) is computed by not averaged classifier
%                   scores.elapse(iternum_index,count_index) is the time [seconds] elapsed in the iteration
%
% Copyright (c) 2014, Yoshitaka Ushiku
% This software is released under the MIT License.
% http://opensource.org/licenses/mit-license.php

if nargin < 7
    codebook = [];
end
if nargin < 8
    iscache = false;
else
    iscache = true;
end
if nargin < 9
    scores.avg = zeros(iternum,count);
    scores.nonavg = zeros(iternum,count);
    scores.elapse = zeros(iternum,count);
end

for ind = 1:count
    obj.mu = [];
    obj.sigma= [];
    [ trainInd, testInd ] = online_learning.split_data( trainingsample, labels );
    for iter_ind = 1:iternum
        if scores.elapse(iter_ind,ind) ~= 0
            continue;
        end
        tic
        if iscache
            % try to load the latest classifiers
            cur_cache_path = [cache_path '_' num2str(ind)];
            if exist([cur_cache_path '_iter' num2str(iter_ind) '.mat'],'file')
                load([cur_cache_path '_iter' num2str(iter_ind) '.mat']);
                disp(['LOADED: ' cur_cache_path '_iter' num2str(iter_ind) '.mat'])
            else
                % learn now
                obj = obj.learn(1,code(:,trainInd),labels(trainInd),codebook,obj.mu,obj.sigma);
                scores.elapse(iter_ind,ind) = toc;
                save([cur_cache_path '_iter' num2str(iter_ind) '.mat'],'obj','-v7.3');
            end
        else
            obj = obj.learn(1,code(:,trainInd),labels(trainInd),codebook,obj.mu,obj.sigma);
            scores.elapse(iter_ind,ind) = toc;
        end
        % evaluating
        score = obj.evaluate_with_test(code(:,testInd),labels(testInd),codebook);
        scores.avg(iter_ind,ind) = score.avg;
        scores.nonavg(iter_ind,ind) = score.nonavg;

        disp(['Iteration: ' num2str(ind) '-' num2str(iter_ind) ' ' sprintf('%.1f(s)',scores.elapse(iter_ind,ind))]);
        disp(['Accuracy of ' obj.type ' (Averaged) is ' sprintf('%2.2f%%',100*score.avg(1))])
        disp(['Accuracy of ' obj.type ' (non-Averaged) is ' sprintf('%2.2f%%',100*score.nonavg(1))])
    end
end

end
