function score = split_evaluation( obj, iternum, count, code, labels, trainingsample, codebook, cache_path )
% Do training multiple iteration and multiple independent times and computes and displays classification accuracy
% Input
%   iternum         the number of iteration per one sequential learning
%   count           the number of independent learning (total iternum * count iterations are performed)
%   code_train      dim * datanum training matrix
%   code_test       dim * test_datanum test data matrix
%   labels_train    datanum vector or classnum * datanum logical matrix
%   labels_test     test_datanum vector of ground truth label
%   codebook        dim * k matrix of PQ codebooks
%                   if empty, samples are regarded as linear feature vector
%   cache_path      (optional) prefix of model cache file name
% Output
%   score           statistics of classification accuracy through learning of count times
%                   score.avg(count_index)[mean,std] is accuracy mean and standard deviation of averaged classifier of final iteration
%                   score.nonavg(count_index)[mean,std] is accuracy mean and standard deviation of not averaged classifier of final iteration
%
% Copyright (c) 2014, Yoshitaka Ushiku
% This software is released under the MIT License.
% http://opensource.org/licenses/mit-license.php


if nargin < 7
    codebook = [];
end
if nargin < 8
    iscache = false;
else
    iscache = true;
end

scores.avg = zeros(1,count);
scores.nonavg = zeros(1,count);
for ind = 1:count
    [ trainInd, testInd ] = online_learning.split_data( trainingsample, labels );
    if iscache
        % try to load the latest classifiers
        cur_cache_path = [cache_path '_' num2str(ind)];
        if exist([cur_cache_path '_iter' num2str(iternum) '.mat'],'file')
            load([cur_cache_path '_iter' num2str(iternum) '.mat']);
        else
            % learn now
            obj = obj.learn(iternum,code(:,trainInd),labels(trainInd),codebook,[],[],cur_cache_path);
        end
    else
        obj = obj.learn(iternum,code(:,trainInd),labels(trainInd),codebook);
    end
    % evaluating
    score = obj.evaluate_with_test(code(:,testInd),labels(testInd));
    scores.avg(ind) = score.avg;
    scores.nonavg(ind) = score.nonavg;
end

% evaluating respectively classifiers w/ averaging and classifiers w/o averaging
score.avg = [mean(100*scores.avg) std(100*scores.avg,1)];
score.nonavg = [mean(100*scores.nonavg) std(100*scores.nonavg,1)];
disp(['Accuracy of ' obj.type ' (Averaged) is ' sprintf('%2.1f',score.avg(1)) ' +/- ' sprintf('%2.1f %%',score.avg(2))])
disp(['Accuracy of ' obj.type ' (non-Averaged) is ' sprintf('%2.1f',score.nonavg(1)) ' +/- ' sprintf('%2.1f %%',score.nonavg(2))])

end
