function varargout = AROW( C, iternum, code, labels, codebook, mu, sigma )
% Adaptive Regularization Of Weight vectors
% Input
%   C               aggressiveness parametor (upper bound of tau)
%   iternum         # of all iteration
%   code            dim * datanum training matrix
%   labels          datanum vector
%   codebook        dim * k matrix of PQ codebooks
%   mu              dim+1 * classnum initial weights (default: zeros)
%   sigma           dim+1 * classnum initial weights (default: zeros)
% Output
%   mu              dim+1 * classnum weights
%   sigma           dim+1 * classnum sigma
%
% Copyright (c) 2014, Yoshitaka Ushiku
% This software is released under the MIT License.
% http://opensource.org/licenses/mit-license.php

labels = online_learning.labvec2labmat(labels);
n = size(labels,1);

% evaluation setting
if nargin < 5 || isempty(codebook)
    isPQ = false;
    d = size(code,1);
else
    isPQ = true;
    d = size(codebook,1);
    G = d / size(code,1);
end

% training and testing
if nargin < 6 || isempty(mu)
    mu = zeros(d+1,n);
    sigma = ones(d+1,n);
end

is_averaging = false;
for iter = 1:iternum

    if iter==iternum && nargout == 3
        is_averaging = true;
        dmu = zeros(size(mu));
    end
    update = 0;

    for trnind = online_learning.randpermrs(size(labels,2))

        if sum(labels(:,trnind))==0; continue; end
        if isPQ
            cb_ind = double(reshape(code(:,trnind*ones(G,1))',d,1))*d + (1:d)';
            F = [codebook(cb_ind); 1];
        else
            F = [code(:,trnind); 1];
        end
        prediction = F'*mu;
        y = find(labels(:,trnind));
        [y_pred,i_pred] = min(prediction(y));
        prediction(y) = -Inf; y = y(i_pred);
        [s_pred,s] = max(prediction);
        hingeloss = max(0,1-(y_pred-s_pred));
        update = update + 1;

        if hingeloss == 0
            continue
        end
        
        F_2 = F.^2;
        c = (sigma(:, y) + sigma(:, s))' * F_2 + 1/C;
        sigma_y_F = sigma(:, y) .* F * (hingeloss/c);
        sigma_s_F = sigma(:, s) .* F * (hingeloss/c);
        mu(:,y) = mu(:,y) + sigma_y_F;
        mu(:,s) = mu(:,s) - sigma_s_F;
        sigma(:,y) = 1./(1./sigma(:,y) + F_2 * C);
        sigma(:,s) = 1./(1./sigma(:,s) + F_2 * C);

        if is_averaging
            dmu(:,y) = dmu(:,y) + update*sigma_y_F;
            dmu(:,s) = dmu(:,s) - update*sigma_s_F;
        end

    end

    fprintf('iter\t%d\t\tupdated %d \033[K\n',iter,size(labels,2));

end

if nargout == 3
    update = update + 1;
    avgmu = mu-dmu/update;
    varargout{1} = avgmu;
    varargout{2} = mu;
    varargout{3} = sigma;
else
    varargout{1} = mu;
    varargout{2} = sigma;
end

end


