function varargout = PAwOVR( C, beta, iternum, code, labels, codebook, mu )
% Averaged Passive-Aggressive algorithm using OVR manner with same amoount
% of pos./neg. samples
% Input
%   C               aggressiveness parametor (upper bound of tau)
%   beta            # neg. sample / # pos. sample
%   iternum         # of all iteration
%   code            dim * datanum training matrix
%   labels          datanum vector or classnum * datanum logical matrix
%   codebook        dim * k matrix of PQ codebooks
%                   if empty, usual PA (but OVR) is performed
%   mu              dim+1 * classnum initial weights (default: zeros)
% Output
%   avgmu           dim+1 * classnum averaged weights
%   mu              dim+1 * classnum weights
%
% Copyright (c) 2014, Yoshitaka Ushiku
% This software is released under the MIT License.
% http://opensource.org/licenses/mit-license.php

labels = online_learning.labvec2labmat(labels);
n = size(labels,1);

% evaluation setting
if nargin < 6 || isempty(codebook)
    isPQ = false;
    d = size(code,1);
else
    isPQ = true;
    d = size(codebook,1);
    G = d / size(code,1);
end

% training and testing
if nargin < 7 || isempty(mu)
    mu = zeros(d+1,n);
end

pos_smpl_num = sum(labels,2)';
hingelosses = zeros(1,n);
is_averaging = false;
for iter = 1:iternum

    if iter==iternum && nargout == 2
        is_averaging = true;
        dmu = zeros(size(mu));
    end
    updates = zeros(1,n);
    neg_smpl_prob = pos_smpl_num * beta ./ (size(labels,2)-pos_smpl_num);

    for trnind = online_learning.randpermrs(size(labels,2))

        if sum(labels(:,trnind))==0; continue; end
        if isPQ
            cb_ind = double(reshape(code(:,trnind*ones(G,1))',d,1))*d + (1:d)';
            F = [codebook(cb_ind); 1];
        else
            F = [code(:,trnind); 1];
        end
        y = sign(labels(:,trnind)-0.5)';
        cur_train = y == 1 | (y == -1 & (rand(1,n) < neg_smpl_prob));
        hingelosses(cur_train) = 1-F'*mu(:,cur_train).*y(cur_train);
        cur_update = hingelosses > 0 & cur_train;

        updates = updates + cur_train;
        taus_F = F*(min(C,hingelosses(cur_update)/(F'*F)).*y(cur_update));

        mu(:,cur_update) = mu(:,cur_update)+taus_F;
        if is_averaging
            dmu(:,cur_update) = dmu(:,cur_update)+bsxfun(@times,updates(cur_update),taus_F);
        end

    end
    
    fprintf('iter\t%d\t\tupdated %.1f \033[K\n',iter,mean(updates));

end

if nargout == 2
    updates = updates + 1;
    avgmu = mu-bsxfun(@rdivide,dmu,updates);
    varargout{1} = avgmu;
    varargout{2} = mu;
else
    varargout{1} = mu;
end

end
