function varargout = SCW( C, eta, iternum, code, labels, codebook, mu, sigma )
% Soft Confidence Weighted
% Input
%   C               aggressiveness parametor (upper bound of update)
%   eta             probability of classifying correctly
%   iternum         # of all iteration
%   code            dim * datanum training matrix
%   labels          datanum vector
%   codebook        dim * k matrix of PQ codebooks
%   mu              dim+1 * classnum initial weights (default: zeros)
%   sigma           dim+1 * classnum initial weights (default: zeros)
% Output
%   avgmu           dim+1 * classnum averaged weights
%   mu              dim+1 * classnum weights
%   sigma           dim+1 * classnum sigma
%
% Copyright (c) 2014, Yoshitaka Ushiku
% This software is released under the MIT License.
% http://opensource.org/licenses/mit-license.php

phi = cdf('Normal',eta,0,1)^-1;

labels = online_learning.labvec2labmat(labels);
n = size(labels,1);

% evaluation setting
if nargin < 6 || isempty(codebook)
    isPQ = false;
    d = size(code,1);
else
    isPQ = true;
    d = size(codebook,1);
    G = d / size(code,1);
end

% training and testing
if nargin < 7 || isempty(mu)
    mu = zeros(d+1,n);
    sigma = ones(d+1,n);
end

is_averaging = false;
for iter = 1:iternum

    if iter==iternum && nargout == 3
        is_averaging = true;
        dmu = zeros(size(mu));
    end
    update = 0;

    for trnind = online_learning.randpermrs(size(labels,2))

        if sum(labels(:,trnind))==0; continue; end
        if isPQ
            cb_ind = double(reshape(code(:,trnind*ones(G,1))',d,1))*d + (1:d)';
            F = [codebook(cb_ind); 1];
        else
            F = [code(:,trnind); 1];
        end
        prediction = F'*mu;
        y = find(labels(:,trnind));
        [y_pred,i_pred] = min(prediction(y));
        prediction(y) = -Inf; y = y(i_pred);
        [s_pred,s] = max(prediction);
        margin = y_pred-s_pred;
        x_sigma_x = (sigma(:, y) + sigma(:, s))' * (F.^2);
        update = update + 1;

        if margin > phi * sqrt(x_sigma_x)
            continue
        end
        
        sub_alpha = sqrt(margin^2*phi^4/4+x_sigma_x*phi^2*(1+phi^2));    
        alpha = (sub_alpha-margin*(1+phi^2/2)) ...
              / ( x_sigma_x*(1 + phi^2) );
        alpha = min(C,max(0,alpha));
        sub_sigma_factor = alpha*x_sigma_x*phi;
        sigma_factor = alpha*phi / ( abs(sqrt(sub_sigma_factor^2+2*x_sigma_x)-sub_sigma_factor) + 2*sub_sigma_factor );

        sigma_y_F = sigma(:, y) .* F;
        sigma_s_F = sigma(:, s) .* F;
        sigma(:,y) = sigma(:,y) - 2*sigma_factor*sigma_y_F.^2;
        sigma(:,s) = sigma(:,s) - 2*sigma_factor*sigma_s_F.^2;

        sigma_y_F = sigma_y_F * alpha;
        sigma_s_F = sigma_s_F * alpha;
        mu(:,y) = mu(:,y) + sigma_y_F;
        mu(:,s) = mu(:,s) - sigma_s_F;
        if is_averaging
            dmu(:,y) = dmu(:,y) + update * (sigma_y_F);
            dmu(:,s) = dmu(:,s) - update * (sigma_s_F);
        end

    end
    
    fprintf('iter\t%d\t\tupdated %d \033[K\n',iter,size(labels,2));

end

if nargout == 3
    update = update + 1;
    avgmu = mu-dmu/update;
    varargout{1} = avgmu;
    varargout{2} = mu;
    varargout{3} = sigma;
else
    varargout{1} = mu;
    varargout{2} = sigma;
end

end
