function varargout = SGDSVMMUL( lambda, iternum, code, labels, codebook, theta, t_zero )
% Averaged SGD SVM using Multiclass-loss [Crammer and Singer, 2001]
% Input
%   lambda          trade-off parameter between the risk and the
%                   regularization penalty
%   iternum         # of all iteration
%   code            dim * datanum training matrix
%   labels          datanum vector or classnum * datanum logical matrix
%   codebook        dim * k matrix of PQ codebooks
%                   if empty, usual PA (but OVR) is performed
%   theta           dim+1 * classnum initial weights (default: zeros)
%   t_zero          initial parameter for the step-size, gamma
% Output
%   avgtheta        dim+1 * classnum averaged weights
%   theta           dim+1 * classnum weights
%
% Copyright (c) 2014, Yoshitaka Ushiku
% This software is released under the MIT License.
% http://opensource.org/licenses/mit-license.php

labels = online_learning.labvec2labmat(labels);
num_class = size(labels,1);
disp('SGD SVM (multiclass)')

% evaluation setting
if nargin < 5 || isempty(codebook)
    fprintf('\tw/o PQ\n')
    isPQ = false;
    dim_feature = size(code,1);
else
    fprintf('\twith PQ\n')
    isPQ = true;
    dim_feature = size(codebook,1);
    G = dim_feature / size(code,1);
end

% training and testing
if nargin < 6 || isempty(theta)
    fprintf('\tw/o initial value\n')
    u = zeros(dim_feature+1,num_class);
else
    fprintf('\twith initial value\n')
    u = theta;
end

% training and testing
if nargin < 7
    t_zero = 1;
end

if lambda > 0
    gamma = 1/(lambda * t_zero);
    fprintf('\twith regularization (lambda=%f,gamma=%f)\n',lambda,gamma)
elseif lambda == 0
    if nargin < 7
        gamma = 0.1;
    else
        gamma = t_zero;
    end
    fprintf('\tw/o regularization (gamma=%f)\n',gamma)
else
    lambda = 1/size(labels,2);
    gamma = 1/(lambda * t_zero);
    fprintf('\twith regularization (lambda=%f,gamma=%f)\n',lambda,gamma)
end
is_averaging = false;
alpha = ones(1,num_class);
for iter = 1:iternum

    if iter==iternum && nargout == 2
        is_averaging = true;
        beta = ones(1,num_class);
        tau = zeros(1,num_class);
        u_hat = zeros(size(u));
        t = 1;
    end

    for trnind = online_learning.randpermrs(size(labels,2))

        if sum(labels(:,trnind))==0; continue; end
        if isPQ
            cb_ind = double(reshape(code(:,trnind*ones(G,1))',dim_feature,1))*dim_feature + (1:dim_feature)';
            x = [codebook(cb_ind); 1];
        else
            x = [code(:,trnind); 1];
        end
        prediction = x'*u./alpha;
        y = find(labels(:,trnind));
        [y_pred,i_pred] = min(prediction(y));
        prediction(y) = -Inf; y = y(i_pred);
        [s_pred,s] = max(prediction);
        hingeloss = max(0,1-(y_pred-s_pred));

        if lambda > 0
            gamma = 1/(1/gamma + lambda);
            % regularization
            alpha = alpha/(1-lambda*gamma);
        end

        if hingeloss == 0
            continue
        end
        
        % update
        u(:,y) = u(:,y) + alpha(y)*gamma*x;
        u(:,s) = u(:,s) - alpha(s)*gamma*x;
        if is_averaging
            t = t+1;
            eta = 1/t;
            beta = beta/(1-eta);
            u_hat(:,y) = u_hat(:,y) - tau(y).*alpha(y).*gamma*x;
            u_hat(:,s) = u_hat(:,s) + tau(s).*alpha(s).*gamma*x;
            tau = tau + eta*beta./alpha;
        end
    end

    fprintf('iter\t%d\t\tupdated %d\033[K\n',iter,size(labels,2));

end

if nargout == 2
    varargout{1} = bsxfun(@rdivide,bsxfun(@times,u,tau)+u_hat,beta);
    varargout{2} = bsxfun(@rdivide,u,alpha);
else
    varargout{1} = bsxfun(@rdivide,u,alpha);
end

end
