function varargout = SGDSVMuOVR( lambda, iternum, code, labels, codebook, mu, t_zero )
% Averaged SGD SVM using OVR manner with all samples
% Input
%   lambda          trade-off parameter between the risk and the
%                   regularization penalty
%   iternum         # of all iteration
%   code            dim * datanum training matrix
%   labels          datanum vector or classnum * datanum logical matrix
%   codebook        dim * k matrix of PQ codebooks
%                   if empty, usual PA (but OVR) is performed
%   mu              dim+1 * classnum initial weights (default: zeros)
%   t_zero          initial parameter for the step-size, eta
% Output
%   avgmu           dim+1 * classnum averaged weights
%   mu              dim+1 * classnum weights
%
% Copyright (c) 2014, Yoshitaka Ushiku
% This software is released under the MIT License.
% http://opensource.org/licenses/mit-license.php

labels = online_learning.labvec2labmat(labels);
n = size(labels,1);

% evaluation setting
if nargin < 5 || isempty(codebook)
    isPQ = false;
    d = size(code,1);
else
    isPQ = true;
    d = size(codebook,1);
    G = d / size(code,1);
end

% training and testing
if nargin < 6 || isempty(mu)
    mu_norm = ones(1,n);
    mu = zeros(d+1,n);
else
    mu_norm = sum(mu.^2,1).^.5;
    mu = bsxfun(@rdivide,mu,mu_norm);
end

% training and testing
if nargin < 7
    t_zero = 1;
end

if lambda > 0
    eta_inv = lambda * t_zero;
elseif lambda == 0
    if nargin < 7
        eta_inv = 10;
    else
        eta_inv = 1/t_zero;
    end
else
    lambda = 1/size(labels,2);
    eta_inv = lambda * t_zero;
end
is_averaging = false;
for iter = 1:iternum

    if iter==iternum && nargout == 2
        is_averaging = true;
        dmu = zeros(size(mu));
        reg_cumprod = ones(1,n);
    end
    update = 0;

    for trnind = online_learning.randpermrs(size(labels,2))

        if sum(labels(:,trnind))==0; continue; end
        if isPQ
            cb_ind = double(reshape(code(:,trnind*ones(G,1))',d,1))*d + (1:d)';
            F = [codebook(cb_ind); 1];
        else
            F = [code(:,trnind); 1];
        end
        y = sign(labels(:,trnind)-0.5)';
        hingelosses = 1-F'*mu.*mu_norm.*y;
        cur_update = hingelosses > 0;

        update = update + 1;

        if lambda > 0
            eta_inv = eta_inv + lambda;
            % regularization
            mu = mu-mu*lambda/eta_inv;
            % averaging
            if is_averaging
                dmu = dmu-update*mu*lambda/eta_inv;
            end
        end
        
        % update
        mu(:,cur_update) = mu(:,cur_update)+F/eta_inv*y(cur_update);
        % averaging
        if is_averaging
            dmu(:,cur_update) = dmu(:,cur_update)+update*F/eta_inv*y(cur_update);
        end

    end
    
    mu_norm = sum(mu.^2,1).^.5;
    mu = bsxfun(@rdivide,mu,mu_norm);
    fprintf('iter\t%d\t\tupdated %d\033[K\n',iter,update);

end

if nargout == 2
    update = update + 1;
    avgmu = mu-dmu/update;
    varargout{1} = avgmu;
    varargout{2} = mu;
else
    varargout{1} = mu;
end

end
