function varargout = SGDSVMwOVR( lambda, beta, iternum, code, labels, codebook, mu, t_zero )
% Averaged SGD SVM using OVR manner with all samples
% Input
%   lambda          trade-off parameter between the risk and the
%                   regularization penalty
%   beta            # neg. sample / # pos. sample
%   iternum         # of all iteration
%   code            dim * datanum training matrix
%   labels          datanum vector or classnum * datanum logical matrix
%   codebook        dim * k matrix of PQ codebooks
%                   if empty, usual PA (but OVR) is performed
%   mu              dim+1 * classnum initial weights (default: zeros)
%   t_zero          initial parameter for the step-size, eta
% Output
%   avgmu           dim+1 * classnum averaged weights
%   mu              dim+1 * classnum weights
%
% Copyright (c) 2014, Yoshitaka Ushiku
% This software is released under the MIT License.
% http://opensource.org/licenses/mit-license.php

labels = online_learning.labvec2labmat(labels);
n = size(labels,1);

% evaluation setting
if nargin < 6 || isempty(codebook)
    isPQ = false;
    d = size(code,1);
else
    isPQ = true;
    d = size(codebook,1);
    G = d / size(code,1);
end

% training and testing
if nargin < 6 || isempty(mu)
    mu_norm = ones(1,n);
    mu = zeros(d+1,n);
else
    mu_norm = sum(mu.^2,1).^.5;
    mu = bsxfun(@rdivide,mu,mu_norm);
end

% training and testing
if nargin < 8
    t_zero = 1;
end

pos_smpl_num = sum(labels,2)';
hingelosses = zeros(1,n);
if lambda > 0
    eta_invs = lambda * t_zero * ones(1,n);
elseif lambda == 0
    if nargin < 7
        eta_invs = 10 * ones(1,n);
    else
        eta_invs = 1/t_zero * ones(1,n);
    end
else
    lambda = 1/size(labels,2);
    eta_invs = lambda * t_zero * ones(1,n);
end
is_averaging = false;
for iter = 1:iternum

    if iter==iternum && nargout == 2
        is_averaging = true;
        dmu = zeros(size(mu));
        reg_cumprod = ones(1,n);
    end
    updates = zeros(1,n);
    neg_smpl_prob = pos_smpl_num * beta ./ (size(labels,2)-pos_smpl_num);

    for trnind = online_learning.randpermrs(size(labels,2))

        if sum(labels(:,trnind))==0; continue; end
        if isPQ
            cb_ind = double(reshape(code(:,trnind*ones(G,1))',d,1))*d + (1:d)';
            F = [codebook(cb_ind); 1];
        else
            F = [code(:,trnind); 1];
        end
        y = sign(labels(:,trnind)-0.5)';
        cur_train = y == 1 | (y == -1 & (rand(1,n) < neg_smpl_prob));
        hingelosses(cur_train) = 1-F'*mu(:,cur_train).*mu_norm(cur_train).*y(cur_train);
        cur_update = hingelosses > 0 & cur_train;

        updates = updates + cur_train;

        if lambda > 0
            eta_invs(cur_train) = eta_invs(cur_train) + lambda;
            % regularization
            cur_dmu = bsxfun(@times,mu(:,cur_train),lambda./eta_invs(cur_train));
            mu(:,cur_train) = mu(:,cur_train)-cur_dmu;
            % averaging
            if is_averaging
                dmu(:,cur_train) = dmu(:,cur_train)-...
                    bsxfun(@times,cur_dmu,updates(cur_train));
            end
        end
        
        % update
        F_eta_invs = F*(y(cur_update)./eta_invs(cur_update));
        mu(:,cur_update) = mu(:,cur_update)+F_eta_invs;
        % averaging
        if is_averaging
            dmu(:,cur_update) = dmu(:,cur_update)+...
                bsxfun(@times,updates(cur_update),F_eta_invs);
        end

    end
    
    mu_norm = sum(mu.^2,1).^.5;
    mu = bsxfun(@rdivide,mu,mu_norm);
    fprintf('iter\t%d\t\tupdated %.1f \033[K\n',iter,mean(updates));

end

if nargout == 2
    updates = updates + 1;
    avgmu = mu-bsxfun(@rdivide,dmu,updates);
    varargout{1} = avgmu;
    varargout{2} = mu;
else
    varargout{1} = mu;
end

end
